//+------------------------------------------------------------------+
//|                                              BK_VT.Grid_v1.2.mq4 |
//|                                                           Victor |
//| v1.2 (2015-05-02):                                               |
//| - improved: removed warning when there are no buy/sell prices    |
//|                                                                  |
//| v1.1 (2015-05-01):                                               |
//| - new: cancel all pending orders at given time                   |
//| - improved: handling of both buy/sell sides at once              |
//|                                                                  |
//| v1.0 (2015-04-25):                                               |
//| - initial version                                                |
//+------------------------------------------------------------------+

/*
Let's say user specifies 3 levels
And he specifie
TP L1 is 10
TP L2 is 10
TP L3 is 12

EURUSD is at 10700
User sets a sell at 10725
The EA instantly places three orders 
Order 1 at 10725
Order 2 at 10750
Order 3 at 10775
Stop on ALL is 10800

If TP on orders 1 is hit then order 2 and 3 are automatically cancelled
If order 2 is entered then TP is moved to 10750
If order 3 is enters the TP is moved to 10763 etc

The multiplication of size is handled by inputs you already provided. 

In FXCM US the TP is automatically moved to the latest TP but for non FIFO accounts you would need to have the EA move them. 
*/

#include <stderror.mqh>
#include <stdlib.mqh>

extern string x01 = "## Trade parameters (SL and TP are in pips)";
extern double sellPriceAboveMarket = 0;
extern double buyPriceBelowMarket = 0;
extern int levels = 3;
extern double startingLot = 0.01;
extern int lot2_multiplier = 4;
extern int lot3_multiplier = 11;
extern int lot4_multiplier = 50;
extern int lot5_multiplier = 1;
extern int lot6_multiplier = 1;
extern int SL = 75;
extern int TP1 = 10;
extern int TP2 = 10;
extern int TP3 = 10;
extern int TP4 = 10;
extern int TP5 = 10;
extern int TP6 = 10;
extern int pipsFromLevel1to2 = 25;
extern int pipsFromLevel2to3 = 25;
extern int pipsFromLevel3to4 = 25;
extern int pipsFromLevel4to5 = 25;
extern int pipsFromLevel5to6 = 25;

extern string x02 = "## Server time in a format 00:00 to 23:59";
extern string x03 = "## Or date in a format yyyy.MM.dd HH:mm";
extern string x04 = "## Date example: 2015.01.31 15:30";
extern string cancelPendingOrdersAt = "";

extern string x05 = "## Other settings";
extern bool sendEmailOnError = false;
extern int magic = 2724621;

double pip;
bool prerequisitesOK = true;

int buy_currentLevel = -1; // -1 = no orders
int buy_marketOrders = 0;
int buy_pendingOrders = 0;
int sell_currentLevel = -1; // -1 = no orders
int sell_marketOrders = 0;
int sell_pendingOrders = 0;

double lot1, lot2, lot3, lot4, lot5, lot6;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init() {
   // Get value of one pip
   pip = getPipValue();
   
   // Check if any order is active (opened by previous launch of the bot)
   if (isAnyOrderActive()) {
      showMessage("[WARN] There is already active pending or market order. Please close it and restart the bot.", false);
      prerequisitesOK = false;
      return(0);
   }

   // Check if cancelPendingOrdersAt is OK
   cancelPendingOrdersAt = StringTrimRight(StringTrimLeft(cancelPendingOrdersAt));
   if (cancelPendingOrdersAt != "") {
      if (StringFind(cancelPendingOrdersAt, ":") == -1) {
         showMessage("[WARN] Wrong cancelPendingOrdersAt format. Missing semicolon as minute separator.", false);
         prerequisitesOK = false;
         return(0);
      }
   }
   
   // Normalize user input
   sellPriceAboveMarket = NormalizeDouble(sellPriceAboveMarket, Digits);
   buyPriceBelowMarket = NormalizeDouble(buyPriceBelowMarket, Digits);

   // Validate price levels
   if (sellPriceAboveMarket <= 0 && buyPriceBelowMarket <= 0) {
      // Do not show warning for "templates"
      Print("[WARN] Please define sell or buy price level.");
      prerequisitesOK = false;
      return(0);
   }
   
   // Get upper and lower prices based on current bid/ask prices and allowed distance
   double upperPrice = Ask + MarketInfo(Symbol(), MODE_STOPLEVEL)*Point;
   double lowerPrice = Bid - MarketInfo(Symbol(), MODE_STOPLEVEL)*Point;
   
   // Validate sell/buy price position
   if (sellPriceAboveMarket > 0 && sellPriceAboveMarket <= upperPrice) {
      showMessage("[WARN] sellPriceAboveMarket is inside market.", false);
      prerequisitesOK = false;
      return(0);
   }
   if (buyPriceBelowMarket >= lowerPrice) {
      showMessage("[WARN] buyPriceBelowMarket is inside market.", false);
      prerequisitesOK = false;
      return(0);
   }   

   drawStatusLabel("", clrNONE);
   buy_currentLevel = -1;
   sell_currentLevel = -1;
   prerequisitesOK = true;
   return(0);
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit() {
   return(0);
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start() {
   // Check if pre-requisites are OK. Validation done in init()
   if (prerequisitesOK == false) {
      drawStatusLabel("Grid: input error", clrRed);
      return(0);
   }

   if (buy_currentLevel == -1 || sell_currentLevel == -1) {
      buy_currentLevel = 0;
      sell_currentLevel = 0;
      drawStatusLabel("Grid: active", clrGreen);
      
      // Setup grid
      for (int i = 0; i < levels; i++) {
         if (buyPriceBelowMarket > 0) {
            openPendingOrder(OP_BUYLIMIT, getBuyPriceByLevel(i), 0, buyPriceBelowMarket-SL*pip, getTPPipsByLevel(i), getLotByLevel(i));
         }
         if (sellPriceAboveMarket > 0) {
            openPendingOrder(OP_SELLLIMIT, getSellPriceByLevel(i), 0, sellPriceAboveMarket+SL*pip, getTPPipsByLevel(i), getLotByLevel(i));
         }
      }
      
      RefreshRates();
      if (buyPriceBelowMarket > 0) {
         buy_marketOrders = getOrderCount(OP_BUY);
         buy_pendingOrders = getOrderCount(OP_BUYLIMIT);
      }
      if (sellPriceAboveMarket > 0) {
         sell_marketOrders = getOrderCount(OP_SELL);
         sell_pendingOrders = getOrderCount(OP_SELLLIMIT);
      }
            
   } else {
      // If there are active orders - manage them
      if (OrdersTotal() > 0) {
         if (isAnyOrderActive()) {
            // === BUY side of the grid ============================================================================================
            if (buyPriceBelowMarket > 0) {           
               int buy_newMarketOrders = getOrderCount(OP_BUY);
               int buy_newPendingOrders = getOrderCount(OP_BUYLIMIT);
               buy_currentLevel = levels-buy_newPendingOrders;
   
               if (buy_newMarketOrders == 0 && buy_marketOrders > 0 && buy_newPendingOrders > 0) {
                  // TP reached
                  Print("[INFO] Profit target reached. Cancelling remaining BUY pending orders...");
                  cancelPendingOrders(OP_BUYLIMIT);
                  
               } else if (buy_newMarketOrders > 0 && buy_marketOrders > 0 && buy_newMarketOrders != buy_marketOrders) {
                  // New market order triggered - ajust TP
                  Print("[INFO] New BUY market order triggered. Adjusting TPs on remaining...");
                  double newBuyTakeProfitPrice = getBuyPriceByLevel(buy_currentLevel-1) + getTPPipsByLevel(buy_currentLevel-1)*pip;
                  adjustTakeProfitOnMarketOrders(newBuyTakeProfitPrice, OP_BUY);          
               }
               buy_marketOrders = buy_newMarketOrders;
               buy_pendingOrders = buy_newPendingOrders;
            }
            
            // === SELL side of the grid ===========================================================================================
            if (sellPriceAboveMarket > 0) {
               int sell_newMarketOrders = getOrderCount(OP_SELL);
               int sell_newPendingOrders = getOrderCount(OP_SELLLIMIT);
               sell_currentLevel = levels-sell_newPendingOrders;
   
               if (sell_newMarketOrders == 0 && sell_marketOrders > 0 && sell_newPendingOrders > 0) {
                  // TP reached
                  Print("[INFO] Profit target reached. Cancelling remaining SELL pending orders...");
                  cancelPendingOrders(OP_SELLLIMIT);
                  
               } else if (sell_newMarketOrders > 0 && sell_marketOrders > 0 && sell_newMarketOrders != sell_marketOrders) {
                  // New market order triggered - ajust TP
                  Print("[INFO] New SELL market order triggered. Adjusting TPs on remaining...");
                  double newSellTakeProfitPrice = getSellPriceByLevel(sell_currentLevel-1) - getTPPipsByLevel(sell_currentLevel-1)*pip;
                  adjustTakeProfitOnMarketOrders(newSellTakeProfitPrice, OP_SELL);          
               }
               sell_marketOrders = sell_newMarketOrders;
               sell_pendingOrders = sell_newPendingOrders;
            }
            
            // Closing time - If the pair is NOT floating a live trade and it is XX time just clear all pending.
            if (cancelPendingOrdersAt != "") {
               if (buy_marketOrders == 0 && sell_marketOrders == 0 && (buy_pendingOrders > 0 || sell_pendingOrders > 0)) {
                  datetime cancelTime = StrToServerTime(cancelPendingOrdersAt);
                  if (TimeCurrent() >= cancelTime) {
                     Print("[INFO] Cancelling all pending orders as expiration time is reached...");
                     cancelPendingOrders(OP_SELLLIMIT);
                     cancelPendingOrders(OP_BUYLIMIT);
                  }
               }
            }               
            
            // Track status of the grid
            if (buy_marketOrders > 0 || sell_marketOrders > 0) {
               if (buyPriceBelowMarket > 0 && sellPriceAboveMarket > 0) {  
                  drawStatusLabel(
                     "Grid: buy level " + buy_currentLevel + " of " + levels 
                        + "; sell level " + sell_currentLevel + " of " + levels, clrGreen);
               } else if (buyPriceBelowMarket > 0) {
                  drawStatusLabel("Grid: level " + buy_currentLevel + " of " + levels, clrGreen);
               } else if (sellPriceAboveMarket > 0) {
                  drawStatusLabel("Grid: level " + sell_currentLevel + " of " + levels, clrGreen);
               }
            }

         } else {
            drawStatusLabel("Grid: finished", clrGray);
         }
      } else {
         drawStatusLabel("Grid: finished", clrGray);
      }
   }

   return(0);
}

void adjustTakeProfitOnMarketOrders(double newTakeProfitPrice, int orderType) {
   newTakeProfitPrice = NormalizeDouble(newTakeProfitPrice, Digits);
   
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != orderType) continue;
      if (newTakeProfitPrice == NormalizeDouble(OrderTakeProfit(), Digits)) continue;
      
      if (!OrderModify(OrderTicket(),OrderOpenPrice(),OrderStopLoss(),newTakeProfitPrice,0)) {
         int error = GetLastError();
         showMessage("[ERROR] Failed to move SL on order: #" + OrderTicket() 
            + ". Error: " + error + ". " + ErrorDescription(error), sendEmailOnError);
      }
   }
}

double getBuyPriceByLevel(int level) {
   double buyPrice = 0;
   if (level == 0) {
      buyPrice = buyPriceBelowMarket;
   } else if (level == 1) {
      buyPrice = buyPriceBelowMarket-pipsFromLevel1to2*pip;
   } else if (level == 2) {
      buyPrice = buyPriceBelowMarket-(pipsFromLevel1to2+pipsFromLevel2to3)*pip;
   } else if (level == 3) {
      buyPrice = buyPriceBelowMarket-(pipsFromLevel1to2+pipsFromLevel2to3+pipsFromLevel3to4)*pip;
   } else if (level == 4) {
      buyPrice = buyPriceBelowMarket-(pipsFromLevel1to2+pipsFromLevel2to3+pipsFromLevel3to4+pipsFromLevel4to5)*pip;
   } else if (level == 5) {
      buyPrice = buyPriceBelowMarket-(pipsFromLevel1to2+pipsFromLevel2to3+pipsFromLevel3to4+pipsFromLevel4to5+pipsFromLevel5to6)*pip;
   }
   return(NormalizeDouble(buyPrice, Digits));
}

double getSellPriceByLevel(int level) {
   double sellPrice = 0;
   if (level == 0) {
      sellPrice = sellPriceAboveMarket;
   } else if (level == 1) {
      sellPrice = sellPriceAboveMarket+pipsFromLevel1to2*pip;
   } else if (level == 2) {
      sellPrice = sellPriceAboveMarket+(pipsFromLevel1to2+pipsFromLevel2to3)*pip;
   } else if (level == 3) {
      sellPrice = sellPriceAboveMarket+(pipsFromLevel1to2+pipsFromLevel2to3+pipsFromLevel3to4)*pip;
   } else if (level == 4) {
      sellPrice = sellPriceAboveMarket+(pipsFromLevel1to2+pipsFromLevel2to3+pipsFromLevel3to4+pipsFromLevel4to5)*pip;
   } else if (level == 5) {
      sellPrice = sellPriceAboveMarket+(pipsFromLevel1to2+pipsFromLevel2to3+pipsFromLevel3to4+pipsFromLevel4to5+pipsFromLevel5to6)*pip;
   }
   return(NormalizeDouble(sellPrice, Digits));
}

double getTPPipsByLevel(int level) {
   double tp = -1;
   if (level == 0) {
      tp = TP1;
   } else if (level == 1) {
      tp = TP2;
   } else if (level == 2) {
      tp = TP3;
   } else if (level == 3) {
      tp = TP4;
   } else if (level == 4) {
      tp = TP5;
   } else if (level == 5) {
      tp = TP6;
   }
   return(tp);
}

double getLotByLevel(int level) {
   double lot;
   if (level == 0) {
      lot = NormalizeDouble(startingLot, 2);
   } else if (level == 1) {
      lot = NormalizeDouble(startingLot * lot2_multiplier, 2);
   } else if (level == 2) {
      lot = NormalizeDouble(startingLot * lot3_multiplier, 2);
   } else if (level == 3) {
      lot = NormalizeDouble(startingLot * lot4_multiplier, 2);
   } else if (level == 4) {
      lot = NormalizeDouble(startingLot * lot5_multiplier, 2);
   } else if (level == 5) {
      lot = NormalizeDouble(startingLot * lot6_multiplier, 2);
   }
   return(lot);
}

bool isAnyPendingOrderActive() {
   bool active = false;
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != OP_BUYLIMIT && OrderType() != OP_BUYSTOP 
            && OrderType() != OP_SELLLIMIT && OrderType() != OP_SELLSTOP) continue;
      active = true;
      break;
   }
   
   return(active);
}

bool isAnyMarketOrderActive() {
   bool active = false;
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != OP_BUY && OrderType() != OP_SELL) continue;
      active = true;
      break;
   }
   
   return(active);
}

int getOrderCount(int orderType) {
   int count = 0;
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != orderType) continue;
      count++;
   }   
   return(count);
}

int getMarketOrderCount() {
   int count = 0;
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != OP_BUY && OrderType() != OP_SELL) continue;
      count++;
   }   
   return(count);
}

int getPendingOrderCount() {
   int count = 0;
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != OP_BUYLIMIT && OrderType() != OP_BUYSTOP 
            && OrderType() != OP_SELLLIMIT && OrderType() != OP_SELLSTOP) continue;
      count++;
   }
   return(count);
}

//+------------------------------------------------------------------+
//| Cancels pending orders opened by this bot.                       |
//+------------------------------------------------------------------+
void cancelPendingOrders(int orderType) {
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != orderType) continue;
      
      cancelPendingOrder(OrderTicket());
   }
}

//+------------------------------------------------------------------+
//| Cancels pending order.                                           |
//+------------------------------------------------------------------+
void cancelPendingOrder(int ticket) {
   if (!OrderSelect(ticket, SELECT_BY_TICKET)) {
      showMessage("Failed to select order by ticket #" + ticket, sendEmailOnError);
      return;
   }

   // Print info about order being cancelled to the log
   OrderPrint();
         
   // Try to delete order multiple times in case of non-critical error
   while(!IsStopped()) {
      bool result = OrderDelete(OrderTicket());

      if (result != true) {
         int error = GetLastError();
         
         switch(error) {
            case ERR_PRICE_CHANGED: Comment("The price has changed. Retrying...");
               RefreshRates();                     // Update data
               continue;                           // Try again on next iteration
            case ERR_OFF_QUOTES: Comment("Off quotes. Waiting for a new tick...");
               while(RefreshRates()==false)        // Up to a new tick
                  Sleep(100);                      // Cycle delay
               continue;                           // Try again on next iteration
            case ERR_TRADE_CONTEXT_BUSY: Comment("Trade context is busy. Retrying...");
               Sleep(500);                         // Simple solution
               RefreshRates();                     // Update data
               continue;                           // Try again on next iteration
            default:
               showMessage("Cannot send order, error: " + error + ". " + ErrorDescription(error), sendEmailOnError);
         }
      }
        
      // Break "while" loop
      break;
   }
  
   // Clear comment
   Comment("");
}

//+------------------------------------------------------------------+
//| Returns active market order ticket if there's one, otherwise -1. |
//+------------------------------------------------------------------+
int getActiveMarketOrderTicket() {
   int ticket = -1;

   // Check if we have active OP_BUY or OP_SELL
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != OP_BUY && OrderType() != OP_SELL) continue;
 
      ticket = OrderTicket();
      break;
   }
   
   return(ticket);
}

bool openPendingOrder(int type, double price, datetime orderExpirationTime, 
      double priceSL, int orderTP, double lot) {
      
   double priceTP;
   int clr;
   bool orderSent = false;
   
   if (type == OP_BUYLIMIT) {
      clr = Blue;
      //priceSL = price - orderSL*pip;
      priceTP = price + orderTP*pip;
   } else if (type == OP_SELLLIMIT) {
      clr = Red;
      //priceSL = price + orderSL*pip;
      priceTP = price - orderTP*pip;
   } else {
      showMessage("Wrong order type! Coding error.", false);
      return(false);
   }

   // Try to repeat send order operation in case of non-critical error
   while (!IsStopped()) {
      int ticket = OrderSend(Symbol(), type, lot, NormalizeDouble(price, Digits), 0, 
            NormalizeDouble(priceSL, Digits), NormalizeDouble(priceTP, Digits), 
            getOrderComment(WindowExpertName()), magic, orderExpirationTime, clr);
             
      if (ticket < 0) {
         int error = GetLastError();
         
         switch(error) {
            case ERR_PRICE_CHANGED: Comment("The price has changed. Retrying...");
               RefreshRates();                     // Update data
               continue;                           // Try again on next iteration
            case ERR_OFF_QUOTES: Comment("Off quotes. Waiting for a new tick...");
               while(RefreshRates()==false)        // Up to a new tick
                  Sleep(100);                       // Cycle delay
               continue;                           // Try again on next iteration
            case ERR_TRADE_CONTEXT_BUSY: Comment("Trade context is busy. Retrying...");
               Sleep(500);                         // Simple solution
               RefreshRates();                     // Update data
               continue;                           // Try again on next iteration
            default:
               // Send email = false as we will open market order
               showMessage("Cannot send pending order, error: " + error + ". " + ErrorDescription(error), false);
               orderSent = false;
         }
      } else {
         orderSent = true;
      }
      
      break;
   }
   
   // Clear comment
   Comment("");
   return(orderSent);
}

//+------------------------------------------------------------------+
//| Opens order                                                      |
//+------------------------------------------------------------------+
void openMarketOrder(int type, int pipsSL, int pipsTP, double pipValue, double orderLots) {
   int clr;
   double price, priceSL, priceTP;
   
   if (type == OP_BUY) {
      clr = Blue;
   } else if (type == OP_SELL) {
      clr = Red;
   } else {
      showMessage("Wrong order type! Coding error.", sendEmailOnError);
      return;
   }

   // Try to repeat send order operation in case of non-critical error
   while(!IsStopped()) {
      if (type == OP_BUY) {
         price = Ask;
         priceSL = NormalizeDouble(Ask - pipsSL*pipValue, Digits);
         priceTP = NormalizeDouble(Ask + pipsTP*pipValue, Digits);
      } else if (type == OP_SELL) {
         price = Bid;
         priceSL = NormalizeDouble(Bid + pipsSL*pipValue, Digits);
         priceTP = NormalizeDouble(Bid - pipsTP*pipValue, Digits);
      }
   
      int ticket = OrderSend(Symbol(), type, orderLots, price, 0, priceSL, priceTP, getOrderComment(WindowExpertName()), magic, 0, clr);  
      if (ticket < 0) {
         int error = GetLastError();
         
         switch(error) {
            case ERR_PRICE_CHANGED: Comment("The price has changed. Retrying...");
               RefreshRates();                     // Update data
               continue;                           // Try again on next iteration
            case ERR_OFF_QUOTES: Comment("Off quotes. Waiting for a new tick...");
               while(RefreshRates()==false)        // Up to a new tick
                  Sleep(100);                      // Cycle delay
               continue;                           // Try again on next iteration
            case ERR_INVALID_PRICE: Comment("Ivalid price. Waiting for a new tick...");
               while(RefreshRates()==false)        // Up to a new tick
                  Sleep(100);                      // Cycle delay
               continue;                           // Try again on next iteration 
            case ERR_TRADE_CONTEXT_BUSY: Comment("Trade context is busy. Retrying...");
               Sleep(500);                         // Simple solution
               RefreshRates();                     // Update data
               continue;                           // Try again on next iteration
            default:
               showMessage("Cannot send order, error: " + error + ". " + ErrorDescription(error), sendEmailOnError);
         }
      }
      
      break;
   }
   
   // Clear comment
   Comment("");
}

//+------------------------------------------------------------------+
//| Value of one pip based on broker digit count                     |
//+------------------------------------------------------------------+
double getPipValue() {
   double p = 0;
   if (Digits==2 || Digits==4) {
      p = Point;
   } else if (Digits==3 || Digits==5) {
      p = 10*Point;
   } else if (Digits==6) {
      p = 100*Point;
   }
   
   return(p);
}

//+------------------------------------------------------------------+
//| Checks whether there is an open order active                     |
//+------------------------------------------------------------------+
bool isAnyOrderActive() {
   bool open = false;
   for (int i = OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == magic) {
         open = true;
         break;
      }
   }
   return(open);
}

//+------------------------------------------------------------------+
//| Converts a MT4-numeric timeframe to its descriptor string        |
//| Usage: string s=TFToStr(15) returns s="M15"                      |
//+------------------------------------------------------------------+
string TFToStr(int tf=0) {
   if (tf == 0)        tf = Period();
   if (tf >= 43200)    return("MN");
   if (tf >= 10080)    return("W1");
   if (tf >=  1440)    return("D1");
   if (tf >=   240)    return("H4");
   if (tf >=    60)    return("H1");
   if (tf >=    30)    return("M30");
   if (tf >=    15)    return("M15");
   if (tf >=     5)    return("M5");
   if (tf >=     1)    return("M1");
   return("");
}

//+------------------------------------------------------------------+
//| Prepares a comment for OrderSend() function.                     |
//| Example: "M15: u825"                                             |
//+------------------------------------------------------------------+
string getOrderComment(string nameOfBot) {
   string tf = TFToStr(Period());
   return(tf + ": " + nameOfBot);
}

//+------------------------------------------------------------------+
//| Shows an Alert with some additional info. If sendMail=true,      | 
//| email will be sent using settings from MT4 options.              |
//+------------------------------------------------------------------+
void showMessage(string text, bool sendEmail) {
   Print(text);
   Alert(Symbol() + " " + WindowExpertName() + ". " + text);
   if (sendEmail && !IsTesting() && !IsOptimization()) {
      string subject = "Error in Account: " + AccountNumber()
         + " (" + WindowExpertName() + ")";
      string body = 
         "Account name: " + AccountName() + "\n" +
         "Account number: " + AccountNumber() + "\n" +
         "Symbol: " + Symbol() + "\n" +
         "Timeframe: " + TFToStr(Period()) + "\n" +
         "Bot: " + WindowExpertName() + "\n" +
         "Magic: " + magic + "\n" +
         "\n" +
         "Error: " + "\n" + 
         text;
      // MT4 bug: have to call GetLastError() before 
      // using SendMail() to clear the error buffer.
      GetLastError();
      SendMail(subject, body);
      int error = GetLastError();
      if (error != ERR_NO_ERROR) {
         string emailError = "Error sending email. Code: " + error 
            + ", description: " + ErrorDescription(error);
         Alert(emailError);
         Print(emailError);
      }
   }
}

//+------------------------------------------------------------------+
//| Returns triggered order ticket if there's one, otherwise -1.     |
//+------------------------------------------------------------------+
int getTriggeredOrderTicket() {
   int ticket = -1;

   // Check if we have active OP_BUY or OP_SELL
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != OP_BUY && OrderType() != OP_SELL) continue;
      
      ticket = OrderTicket();
      break;
   }
   
   return(ticket);
}

//+------------------------------------------------------------------+
//| Returns triggered order type if there's one, otherwise -1.       |
//+------------------------------------------------------------------+
int getTriggeredOrderType(int ticket) {
   int triggeredOrderType = -1;
   if (ticket != -1) {
      if (OrderSelect(ticket, SELECT_BY_TICKET)) {
         triggeredOrderType = OrderType();
      } else {
         showMessage("Failed to select order #" + ticket, sendEmailOnError);
      }
   }
   
   return(triggeredOrderType);
}

//+------------------------------------------------------------------+
//| Cancels opposite pending order:                                  |
//| - if triggeredOrderType is OP_BUY - cancel OP_SELLLIMIT          |
//| - if triggeredOrderType is OP_SELL - cancel OP_BUYLIMIT          |
//| Return:                                                          |
//| - true if opposite pending order is found                        |
//+------------------------------------------------------------------+
bool cancelOppositePendingOrder(int triggeredOrderType) {
   bool oppositeOrderFound = false;
      
   // Check if we have active OP_BUY or OP_SELL
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;

      bool isOpposite = 
         (triggeredOrderType == OP_BUY && OrderType() == OP_SELLLIMIT)
         || (triggeredOrderType == OP_SELL && OrderType() == OP_BUYLIMIT);
         
      if (isOpposite) {
         oppositeOrderFound = true;
   
         // Print info about order being cancelled to the log
         OrderPrint();
         
         // Try to delete order multiple times in case of non-critical error
         while(!IsStopped()) {
            bool result = OrderDelete(OrderTicket());

            if (result != true) {
               int error = GetLastError();
         
               switch(error) {
                  case ERR_PRICE_CHANGED: Comment("The price has changed. Retrying...");
                     RefreshRates();                     // Update data
                     continue;                           // Try again on next iteration
                  case ERR_OFF_QUOTES: Comment("Off quotes. Waiting for a new tick...");
                     while(RefreshRates()==false)        // Up to a new tick
                        Sleep(100);                      // Cycle delay
                     continue;                           // Try again on next iteration
                  case ERR_TRADE_CONTEXT_BUSY: Comment("Trade context is busy. Retrying...");
                     Sleep(500);                         // Simple solution
                     RefreshRates();                     // Update data
                     continue;                           // Try again on next iteration
                  default:
                     showMessage("Cannot send order, error: " + error + ". " + ErrorDescription(error), sendEmailOnError);
               }
            }
           
            // Break "while" loop
            break;
         }
         
         // Break "for" loop
         break;
      }
   }
   
   // Clear comment
   Comment("");
   
   return(oppositeOrderFound);
}

//+------------------------------------------------------------------+
//| Closes all market orders opened by this bot, which were opened   | 
//| prior to given time.                                             |
//+------------------------------------------------------------------+
void closeAllMarketOrders(datetime olderThan) {
   for (int i=OrdersTotal()-1; i >= 0; i--) {
      if (!OrderSelect(i, SELECT_BY_POS)) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != magic) continue;
      if (OrderType() != OP_BUY && OrderType() != OP_SELL) continue;
      if (OrderOpenTime() > olderThan) continue;
      
      closeMarketOrder(OrderTicket(), OrderLots());
   }
}

//+------------------------------------------------------------------+
//| Returns true if order closed, false otherwise                    |
//+------------------------------------------------------------------+
bool closeMarketOrder(int ticket, double lots) {
   int retries = 0;
   bool result = false;

   while(!IsStopped()) {
      while(IsTradeContextBusy()) Sleep(100);
      RefreshRates();
      if (IsConnected() == true) {
         if (!OrderSelect(ticket, SELECT_BY_TICKET)) continue;
         int type = OrderType();
         switch(type) {
            case OP_BUY : result = OrderClose(ticket, NormalizeDouble(lots, 2), MarketInfo(OrderSymbol(), MODE_BID), 10, Goldenrod); break;
            case OP_SELL: result = OrderClose(ticket, NormalizeDouble(lots, 2), MarketInfo(OrderSymbol(), MODE_ASK), 10, Goldenrod); break;
         }
         
         if (result == true) {
            break;
         } else {
            int err = GetLastError();
            Print("Failed to close order: #" + ticket + ". Error: " + err + ". " + ErrorDescription(err));
            Sleep(1000);
            retries += 1;
            if (retries > 15) {
               showMessage("Unable to close order #" + ticket + ". Tried 15 times. Please do it manually.", sendEmailOnError);
               return(false);
            }
         }
      }
   }
  
   return(result);
}

//+------------------------------------------------------------------+
//| Converts given time in a format of HH:MM into a server time,     |
//| taking current date from TimeCurrent(). If date and time are     |
//| given as an input, simply converts it using StrToTime.           |
//+------------------------------------------------------------------+
datetime StrToServerTime(string time) {
   datetime serverTime;
   if (StringFind(time, ".") == -1) {
      serverTime = StrToTime(TimeToStr(TimeCurrent(), TIME_DATE) + " " + time);
   } else {
      serverTime = StrToTime(time);
   }
   
   return(serverTime);
}

void drawStatusLabel(string text, color textColor) {
   string label = "VT_STATUS";
   ObjectCreate(label, OBJ_LABEL, 0, 0, 0);
   ObjectSet(label, OBJPROP_CORNER, 3);
   ObjectSet(label, OBJPROP_XDISTANCE, 5);
   ObjectSet(label, OBJPROP_YDISTANCE, 5);
   ObjectSetText(label, text, 8, "Tahoma", textColor);
}